import os

SPLIT = '+'+'-'*77+'+'


class JLSysInfo:

    def __init__(self):
        pass

    def nvidia_smi(self):
        lines = [SPLIT, '|' + '显卡信息'.center(73,' ')+'|', SPLIT]
        smi = os.popen('nvidia-smi 2>&1').read().split('\n')
        if len(smi) == 2:
            smi[0] = f'|  {smi[0][:75].ljust(75)}|'
            smi[1] = SPLIT
        else:
            smi = [line for line in smi if line.strip()][1:]

        return lines + smi

    def free_h(self):
        lines = [SPLIT, '|' + '系统内存'.center(73,' ')+'|', SPLIT]
        for line in os.popen("free -h").read()[5:].replace('：   ','：').replace('available', 'available ').split('\n'):
            if line.strip():
                if len(line) > 76:
                    line = line[0:76]
                lines.append('| ' + line.ljust(73, ' ')+ '|')

        lines.append(SPLIT)
        return lines

    def show(self, smi=True, mem=True):
        if smi:
            for l in self.nvidia_smi():
                print(l)
        if mem:
            for l in self.free_h():
                print(l)


if __name__ == "__main__":
    sysinfo = JLSysInfo()
    sysinfo.show()




import os


def read_training_logs(dataset_id, configuration, fold):
    """

    :param results_pa"D:\nnUNet\nnUNet_results\"
    :param dataset_id: 数字 Dataset006_Lung的dataset_id就为6
    :param configuration: 训练模式，可选的有[2d | 3d_lower | 3d_fullres | 3d_cascade_fullres]
    :param fold: 交叉验证数量
    :return:
    """
    results_path = os.environ.get("nnUNet_results")
    raw_path = os.environ.get("nnUNet_raw")
    datasets = os.listdir(results_path)

    dataset = ''
    for d in datasets:
        if str(dataset_id).zfill(3) in d:
            dataset = d

    if dataset == '':
        exit(f"没有找到数据集：dataset_id={str(dataset_id).zfill(3)}")

    logs_path = os.path.join(results_path, dataset, f"nnUNetTrainer__nnUNetPlans__{configuration}", f"fold_{fold}")
    print(os.listdir(logs_path))


def _find_last_epoch_from_logfile(file):
    pass


if __name__ == "__main__":
    read_training_logs(6, "3d_fullres", 5)



import json
import requests


def group_notify(key, msg={"msgtype": "text", "text": { "content": "测试消息"}}):
    url = f'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key={key}'
    headers = {"Content-Type": "application/json"}
    return requests.post(url, headers=headers, data=json.dumps(msg))


def generate_nnunet_template(total_epoch, current_epoch, train_loss, val_loss, pseudo_dice, learning_rate, epoch_time, best_dice):
    """ 使用Markdown风格发送
    Usage：(nnunetv2.training.nnUNetTrainer.nnUNetTrainer.py)
        from jilei.wxwork import generate_nnunet_template, group_notify
        msg = generate_nnunet_template(total_epoch=self.num_epochs, current_epoch=self.current_epoch,
                                       train_loss=train_loss, val_loss=val_loss, pseudo_dice=pseudo_dice,
                                       learning_rate=np.round(self.optimizer.param_groups[0]['lr'], decimals=5),
                                       epoch_time=epoch_time,
                                       best_dice=np.round(self._best_ema, decimals=4) if is_best_ema else None)
    """
    markdown = f"# <font color='red'>NEW BEST DICE: {best_dice}!</font>\n" if best_dice else ""
    markdown = markdown + f"**Epoch No.{current_epoch}** / {total_epoch}\n"
    markdown = markdown + f"**train_loss**:     {train_loss}\n"
    markdown = markdown + f"**var_loss**:       {val_loss}\n"
    markdown = markdown + f"**pseudo_dice**:    {pseudo_dice}\n"
    markdown = markdown + f"**learning_rate**:  {learning_rate}\n"
    markdown = markdown + f"{epoch_time}s"
    return {
        "msgtype": "markdown",
        "markdown": {
            "content": markdown
            }
    }






import docker


class JLDocker:
    def __init__(self, env_file=None):
        if not env_file:
            self.client = docker.from_env()

    def __get_pretty_ports_output(self, ports):
        pretty_ports = []
        for cont_port, host_port in ports.items():
            if host_port:
                ip, port = host_port[0]['HostIp'].replace('0.0.0.0', '*'), host_port[0]['HostPort']
                host_port =  f"{ip}:{port}->"
            pretty_ports.append(host_port+cont_port)

        # ", ".join(pretty_ports)
        return pretty_ports

    def ps(self):

        # 猜测网络名, 默认使用bridge. [#docker network ls]
        # 由docker-compose创建的一般名为docker_default或services_default之类，除非在docker-compose.yml中自定义
        networks = [n.name for n in self.client.networks.list()]
        if "bridge" in networks:
            Network = "bridge"

        for network in networks:
            if "_default" in network:
                Network = network

        for c in self.client.containers.list(all=all):
            for network in c.attrs['NetworkSettings']['Networks'].keys():
                Network = network
                break

        assert Network is not None

        None2EmptyList = lambda s: [] if not s else s

        containers = []
        for c in self.client.containers.list(all=all):
            name = c.name[:12].ljust(14)
            image = c.image.tags[0][:12].ljust(14)
            status =  "\033[1;32m{}\033[0m".format(c.status) if "runnin" in c.status else c.status
            cmd = " ".join(None2EmptyList(c.attrs['Config']['Cmd'])+None2EmptyList(c.attrs['Config']['Entrypoint']))[:38].ljust(40)
            ip = c.attrs['NetworkSettings']['Networks'][Network]['IPAddress'][:12].ljust(14)
            ports = ", ".join(self.__get_pretty_ports_output(c.ports))[:56].ljust(50)
            containers.append(f"|  {name}|  {image}|  {ip}|  {status}  |  {cmd}|  {ports}|")

        SPLIT = "-"*160
        titles = ["name".ljust(14),
                "image".ljust(14),
                "ip address".ljust(14),
                "status".ljust(9),
                "command".ljust(40),
                "ports".ljust(50)]
        HEADER = "|  "+"|  ".join(titles) +"|"

        print(SPLIT)
        print(HEADER)
        print("="*160)
        for c in containers:
            print(c)
        print(SPLIT)



if __name__ == "__main__":
    dock = JLDocker()
    dock.ps()



import setuptools

setuptools.setup(
    name="jilei",
    version="0.1.0",
    author="jilei",
    author_email="developer@jlzn.cc",
    description="For internal use.",
    long_description="python setup.py bdist_wheel\ntwine upload ./dist/*",
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)